==> visitors-8.x-2.17.tar.gz: visitors/src/Service/DeviceService.php <==
<?php

namespace Drupal\visitors\Service;

use DeviceDetector\ClientHints;
use DeviceDetector\DeviceDetector;
use Drupal\Core\Database\Connection;
use Drupal\visitors\VisitorsDeviceInterface;

/**
 * Detects the decive type.
 *
 * @package Drupal\visitors\Service
 */
class DeviceService implements VisitorsDeviceInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * DeviceService constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public function getUniqueUserAgents(): array {
    $results = $this->database->select('visitors', 'v')
      ->fields('v', ['visitors_user_agent'])
      ->condition('bot', NULL, 'IS NULL')
      ->distinct()
      ->orderBy('visitors_user_agent')
      ->execute()
      ->fetchAll(\PDO::FETCH_COLUMN, 0);

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeviceDetector(string $user_agent, array $server = NULL): DeviceDetector {
    $client_hints = NULL;
    if ($server) {
      $client_hints = ClientHints::factory($server);
    }

    $dd = new DeviceDetector($user_agent, $client_hints);
    $dd->parse();

    return $dd;
  }

  /**
   * {@inheritdoc}
   */
  public function bulkUpdate(string $user_agent): int {
    $dd = new DeviceDetector($user_agent);
    $dd->parse();
    $fields = [];
    $fields['config_browser_engine'] = $dd->getClient('engine');
    $fields['config_browser_name'] = $dd->getClient('name');
    $fields['config_browser_version'] = $dd->getClient('version');
    $fields['config_client_type'] = $dd->getClient('type');
    $fields['config_device_brand'] = $dd->getBrandName();
    $fields['config_device_model'] = $dd->getModel();
    $fields['config_device_type'] = $dd->getDeviceName();
    $fields['config_os'] = $dd->getOs('short_name');
    $fields['config_os_version'] = $dd->getOs('version');
    $fields['bot'] = (int) $dd->isBot();

    $count = $this->database->update('visitors')
      ->fields($fields)
      ->condition('visitors_user_agent', $user_agent)
      ->condition('bot', NULL, 'IS NULL')
      ->execute();

    return $count;
  }

}

==> visitors-8.x-2.18.tar.gz: visitors/src/Service/DeviceService.php <==
<?php

namespace Drupal\visitors\Service;

use DeviceDetector\ClientHints;
use DeviceDetector\DeviceDetector;
use Drupal\Core\Database\Connection;
use Drupal\visitors\VisitorsDeviceInterface;

/**
 * Detects the decive type.
 *
 * @package Drupal\visitors\Service
 */
class DeviceService implements VisitorsDeviceInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * DeviceService constructor.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public function getUniqueUserAgents(): array {
    $results = $this->database->select('visitors', 'v')
      ->fields('v', ['visitors_user_agent'])
      ->condition('bot', NULL, 'IS NULL')
      ->distinct()
      ->orderBy('visitors_user_agent')
      ->execute()
      ->fetchAll(\PDO::FETCH_COLUMN, 0);

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function getDeviceDetector(string $user_agent, array $server = NULL): DeviceDetector {
    $client_hints = NULL;
    if ($server) {
      $client_hints = ClientHints::factory($server);
    }

    $dd = new DeviceDetector($user_agent, $client_hints);
    $dd->parse();

    return $dd;
  }

  /**
   * {@inheritdoc}
   */
  public function bulkUpdate(string $user_agent): int {
    $dd = new DeviceDetector($user_agent);
    $dd->parse();
    $fields = [];
    $fields['config_browser_engine'] = $dd->getClient('engine');
    $fields['config_browser_name'] = $dd->getClient('short_name');
    $fields['config_browser_version'] = $dd->getClient('version');
    $fields['config_client_type'] = $dd->getClient('type');
    $fields['config_device_brand'] = $dd->getBrandName();
    $fields['config_device_model'] = $dd->getModel();
    $fields['config_device_type'] = $dd->getDeviceName();
    $fields['config_os'] = $dd->getOs('short_name');
    $fields['config_os_version'] = $dd->getOs('version');
    $fields['bot'] = (int) $dd->isBot();

    $count = $this->database->update('visitors')
      ->fields($fields)
      ->condition('visitors_user_agent', $user_agent)
      ->condition('bot', NULL, 'IS NULL')
      ->execute();

    return $count;
  }

}
